#!/usr/local/bin/perl -w

##############################################################
# syslog2snmp.pl, v 0.01 2005/12/20 
#
# syslog2snmp.pl -   	Monitors the CactiThold Log file for new
#                       entries, and when a new entry is detected,
#                       syslog2snmp forwards that message in an
#                       snmp trap (using Net SNMP) to the Harris
#			NetBoss Network Manager.
#
# Author - Chadd Christiansen, Data Network Engineer - FDN Communications.
#
# Created for FDN Communications, and is thereby sole property of FDN.
#
# Copyright: none.
# ############################################################
# syslog2snmp.pl is free software; you can redistribute it and/or modify 
# as you wish.
# 
# syslog2snmp.pl is also a work in progress, so as I find the need 
# to add additional functionallity, the script will grow/mutate. 
# 
# syslog2snmp.pl is distributed in the hope that it will be useful, 
# but WITHOUT ANY WARRANTY; without even the implied warranty of 
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
# 
# The author of this software, assumes no responsibility for any 
# losses incured by use of syslog2snmp, or any part of syslog2snmp. 
#
# USE AT YOUR OWN RISK! 
# 
##############################################################

use strict;


#########
## MAIN #
#########

#####################
## Global Variables #
#####################

my $config = "/usr/local/bin/script_box/syslog2snmp.conf"; # full path to config file
my %Config_Prefs = ();

## Check for Cacti process completion:

my @ProcessList = `ps -U cactiuser`;

while (@ProcessList ne 0) {
	@ProcessList = `ps -U cactiuser`;
	splice(@ProcessList, 0,1);
	sleep (1);
#	print "waiting for Cacti to complete poll...\n";  ## for testing
}

## Open the syslog and send each line as a trap to the SNMP Manager:

&Syslog_to_SNMPtrap;


#############################
## Config Parser Subroutine #
#############################

sub Config_Parser {
        my ($CalledPref) = @_;
	open (CONFIG, "<$config") or die ("Cannot open $config: $!");
	while (<CONFIG>) {
        	chomp;
        	s/#.*//;                # no newline
        	s/^\s+//;               # no comments
        	s/\s+$//;               # no leading white space
        	next unless length;     # no trailing white space
        	my ($var, $value) = split(/\s*=\s*/, $_, 2); 
        	$Config_Prefs{$var} = $value;
	}
	close (CONFIG);

	return $Config_Prefs{$CalledPref};
}

###################################
## Syslog to SNMP trap Subroutine #
###################################

sub Syslog_to_SNMPtrap {

	my $syslog = Config_Parser("PATH TO CACTI SYSLOG FILE");
	my $path_to_snmptrap = Config_Parser("PATH TO SNMPTRAP");
	my $snmp_community = Config_Parser("SNMP COMMUNITY");
	my $snmp_trapreceiver = Config_Parser("TRAP RECEIVER");
	my $snmp_oid = Config_Parser("OID");
	my $snmp_generictrap = Config_Parser("SNMP GENERIC TRAP");
	my $snmp_specifictrap = Config_Parser("SNMP SPECIFIC TRAP");

	open LOG, "+<$syslog" or die "Cannot Open $syslog: $!";
	my @raw_syslog = <LOG>;

	foreach my $line (@raw_syslog) {
		chomp $line;
		my @SplitLine = split(/:/, $line);
		my $alert = "$SplitLine[3]";
		$alert =~ s/^ +//;

		## Send Trap:
#		print "$alert\n"; ## for testing
		
		if ($alert) {

			`$path_to_snmptrap/snmptrap -v 1 -c $snmp_community $snmp_trapreceiver $snmp_oid $snmp_generictrap $snmp_specifictrap '' $snmp_oid $snmp_generictrap s "[THRESHOLD ALERT]: $alert"`;
		} else { print "No alerts detected\n"; }

	}
	## Seek to the beginning of the file and trucate to zero length

	seek LOG, 0, 0;
	truncate LOG, 0;

	close (LOG);
}
